﻿using System;
using Consulenza.ReportWriter.Business;
using Consulenza.ReportWriter.Business.OBJ_PDF;
using System.Data;
using Consulenza.ReportCommon;
using Consulenza.ReportWriter.Business.CHART_PDF;
using Consulenza.ReportWriter.Business.Entity;
using System.Collections.Generic;
using System.Linq;

namespace Consulenza.ReportWriter.Manager.Section.Unica
{
    /// <summary>
    /// S21.PatrimonioFideuramRappresentazionePerEmittenti idSezione =64
    /// </summary>
    public class S21 : Entity.Section
    {


        public S21(EnvironmentFacade environmentFacade, int idSection)
            : base(environmentFacade, idSection)
        {
            try
            {

                Draw();
            }
            catch (Exception ex)
            {
                SectionLogger.Write("S21", ex.Message, SectionLoggerMessageLevel.E, EnvironmentFacade.ReportEnvironment);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        protected override sealed void Draw()
        {
            var dati = GetDataSet();


            #region tipologiaEmittente
            AddElement(new SpacePDF(20));
            if (presenzaGraficoBarre() && dati.Tables["tipologiaEmittente"].Rows.Count > 0)
            {
                #region Tabella titoli obbligazionari

                AddElement(new FormattedTextAreaPDF(getTesto1(), EnvironmentFacade.RendererFacade.XLeftLimit) { FontSize = 7, TextHorizontalAlign = ceTe.DynamicPDF.TextAlign.Justify });
                AddElement(new SpacePDF(15));
                var tabellaTitoliObbligazionari = new TablePDF(EnvironmentFacade.RendererFacade.XLeftLimit, dati.Tables["patrimonio"])
                {
                    Style = Style.ConsulenzaUnica,
                    Header = false,
                    Footer = false,
                    AlternateRow = false,
                    RowHeight = 25
                };
                tabellaTitoliObbligazionari.Columns.Add(new ColumnPDF("descrizione", 190, HorizontalAlignmentType.Sinistra, true, true, 7, ColumnType.Testo, "descrizione", string.Empty));
                tabellaTitoliObbligazionari.Columns.Add(new ColumnPDF("valore", 330, HorizontalAlignmentType.Sinistra, false, false, 7, ColumnType.Testo, "valore", string.Empty) { PaddingLeft = 5 });

                AddElement(tabellaTitoliObbligazionari);
                AddElement(new SpacePDF(25));

                #endregion

                #region Grafico a barre
                if (presenzaGraficoBarre() && dati.Tables[1].Rows.Count > 0)
                {
                    //AddElement(new FormattedTextAreaPDF("Tipologia Emittente", EnvironmentFacade.RendererFacade.XLeftLimit, 100) { BackGroundColor = ColorPDF.Standard_Grigio_SfondoColonnaTabella, FontBold = true, FontSize = 8, FontColor = ColorPDF.ConsulenzaUnica_Rosso, AutoIncrementYWritable = false, BackGroundMarginLeft = 5 });
                    //AddElement(new FormattedTextAreaPDF(string.Format("Distribuzione ({0})", isControvalore() ? "€" : "%"), EnvironmentFacade.RendererFacade.XLeftLimit + 110, 410) { BackGroundColor = ColorPDF.Standard_Grigio_SfondoColonnaTabella, FontBold = true, FontSize = 8, FontColor = ColorPDF.ConsulenzaUnica_Rosso, BackGroundMarginLeft = 5 });
                    //AddElement(new SpacePDF(10));


                    var graficoBarre = new StackedPDF(EnvironmentFacade.RendererFacade.XLeftLimit + 110, 1F)
                    {
                        HeightSingleBar = 25,
                        Width = 400,
                        LabelFormatAxisY = isControvalore() ? FormatType.Intero : FormatType.Decimale2,
                        ShowLineAxisX = true,
                        PageBreak = true,
                        DinamicFloatXEtichettaLateraleBarra = true,
                        MinorGridAxisY = true,
                        RemoveWhiteSpaceToTheRightAxisX = true,
                        RemoveWhiteSpaceBetweenChartAndAxisX = true,
                        MarginAxisY = 0,
                        MarginAxisYFromTop = 0
                    };
                    graficoBarre.Headers.Add(new FormattedTextAreaPDF("Tipologia Emittente", EnvironmentFacade.RendererFacade.XLeftLimit, 100) { BackGroundColor = ColorPDF.Standard_Grigio_SfondoColonnaTabella, FontBold = true, FontSize = 7, FontColor = ColorPDF.ConsulenzaUnica_Rosso, AutoIncrementYWritable = false, BackGroundMarginLeft = 5 });
                    graficoBarre.Headers.Add(new FormattedTextAreaPDF(string.Format("Distribuzione ({0})", isControvalore() ? "€" : "%"), EnvironmentFacade.RendererFacade.XLeftLimit + 110, 410) { BackGroundColor = ColorPDF.Standard_Grigio_SfondoColonnaTabella, FontBold = true, FontSize = 7, FontColor = ColorPDF.ConsulenzaUnica_Rosso, BackGroundMarginLeft = 5 });

                    var dtGraficoBarre = dati.Tables["tipologiaEmittente"].AsEnumerable().OrderBy(r => r.Field<int>("Ordinamento")).CopyToDataTable();
                    var serieCollezione = new List<Serie> { new Serie { Name = "TipologiaEmittente", Border = true } };

                    foreach (DataRow item in dtGraficoBarre.Rows)
                    {
                        serieCollezione[0].Points.Add(new Point
                        {
                            ShowLabelAxisY = true,
                            LabelAxisY = isControvalore() ? Helper.FormatCurrency(item["Controvalore"].ToString()) : Helper.FormatDecimal(item["Percentuale"].ToString(), 2),
                            FontSizeLabelAxisY = 7,

                            Color = new ColorPDF(Convert.ToInt32(item["Red"]), Convert.ToInt32(item["Green"]), Convert.ToInt32(item["Blue"])),
                            Value = isControvalore() ? (Convert.ToDouble(item["Controvalore"]) == 0 ? 0.01 : Convert.ToDouble(item["Controvalore"])) : Convert.ToDouble(item["Percentuale"]) == 0 ? 0.01 : Convert.ToDouble(item["Percentuale"]), // Convert.ToDouble(Helper.FormatPercentage(Convert.ToDecimal(item["Percentuale"]), 2)),

                            ShowLabelAxisX = true,
                            LabelAxisX = item["TipologiaEmittente"].ToString(),
                            FontSizeLabelAxisX = 7,
                            FontBoldLabelAxisY = false
                        });
                    }

                    graficoBarre.SeriesCollection = serieCollezione;

                    AddElement(graficoBarre);
                    AddElement(new SpacePDF(20));
                }

                if (GetNote1().Length > 0)
                {
                    AddElement(new FormattedTextAreaPDF(GetNote1(), EnvironmentFacade.RendererFacade.XLeftLimit) { FontSize = 6, TextHorizontalAlign = ceTe.DynamicPDF.TextAlign.Justify });
                    AddElement(new SpacePDF(20));
                }

                #endregion

            }
            #endregion

            #region prodottiPerEmittente
            if (presenzaTabella() && dati.Tables["prodottiPerEmittente"].Rows.Count > 0)
            {
                #region Testo introduttivo tabella prodotti

                AddElement(new SpacePDF(15));
                var testoTabellaProdotti = Helper.ReplaceVariables(getTesto2(), EnvironmentFacade.ReportEnvironment);

                AddElement(new FormattedTextAreaPDF(testoTabellaProdotti, EnvironmentFacade.RendererFacade.XLeftLimit) { FontSize = 7, TextHorizontalAlign = ceTe.DynamicPDF.TextAlign.Justify });
                AddElement(new SpacePDF(15));

                #endregion
                #region Tabella Prodotti X Emittente

                var tabellaProdottiPerArea = new TablePDF(EnvironmentFacade.RendererFacade.XLeftLimit, dati.Tables["prodottiPerEmittente"], dati.Tables["prodottiPerEmittenteFooter"])
                {
                    Style = Style.ConsulenzaUnica,
                    Header = true,
                    Footer = true,
                    AlternateRow = false,
                    HeaderHeight = 30,
                    HeaderMargin = 2
                };

                #region Columns e FooterColumns

                tabellaProdottiPerArea.Columns.Add(new ColumnPDF("Immagine", 15 + 150, HorizontalAlignmentType.Sinistra, false, false, 7, ColumnType.Objectpdf, string.Empty, "Obbligazione") { HeaderFontSize = 7, FontSize = 7, DeltaYContent = 4, PaddingLeft = 2 });
                //tabellaProdottiPerArea.Columns.Add(new ColumnPDF("Prodotto", 150, HorizontalAlignmentType.Sinistra, false, false, 7, ColumnType.Testo, "Prodotto", "Obbligazione"));
                tabellaProdottiPerArea.Columns.Add(new ColumnPDF("Emittente", 150 + 65, HorizontalAlignmentType.Sinistra, false, false, 7, ColumnType.Testo, "Emittente", "Emittente") { HeaderFontSize = 7, FontSize = 7 });
                tabellaProdottiPerArea.Columns.Add(new ColumnPDF("Controvalore", 100, HorizontalAlignmentType.Sinistra, false, true, 7, ColumnType.Decimale, "Controvalore", "Controvalore attuale (€)") { HeaderFontSize = 7, FontSize = 7, PaddingRight = 4 });
                tabellaProdottiPerArea.Columns.Add(new ColumnPDF("Peso", 40, HorizontalAlignmentType.Sinistra, false, true, 7, ColumnType.Decimale, "Peso", "Peso (%)") { HeaderFontSize = 7, FontSize = 7, PaddingRight = 4 });

                tabellaProdottiPerArea.FooterColumns.Add(new ColumnPDF("Prodotto", 380, HorizontalAlignmentType.Sinistra, true, true, 7, ColumnType.Testo));
                tabellaProdottiPerArea.FooterColumns.Add(new ColumnPDF("Controvalore", 100, HorizontalAlignmentType.Destra, true, true, 7, ColumnType.Decimale) { PaddingRight = 4 });
                tabellaProdottiPerArea.FooterColumns.Add(new ColumnPDF("Peso", 40, HorizontalAlignmentType.Destra, true, true, 7, ColumnType.Decimale) { HeaderPaddingLeft = 3 });

                int a = 0;
                foreach (var item in dati.Tables[3].Rows)
                {
                    tabellaProdottiPerArea.Cells[2, a].HorizontalAlignment = HorizontalAlignmentType.Destra;
                    tabellaProdottiPerArea.Cells[3, a].HorizontalAlignment = HorizontalAlignmentType.Destra;
                    a++;
                }

                #endregion

                #region Cells

                int i = 0;
                foreach (DataRow row in dati.Tables["prodottiPerEmittente"].Rows)
                {
                    if (Convert.ToBoolean(row["IsTotal"]))
                    {
                        // Sfondo grigio per la riga identificata con IsTotal [quella di intestazione di Emittente]
                        for (int c = 0; c < tabellaProdottiPerArea.Columns.Count; c++)
                        {
                            tabellaProdottiPerArea.Cells[c, i].BackgroundColor = ColorPDF.Standard_Grigio_SfondoColonnaTabella;
                        }
                        //tabellaProdottiPerArea.Cells[0, i].ValueObject = new RectanglePDF(10, 10, new ColorPDF(row["CodiceHexEmittente"].ToString()));
                        var listaOggetti = new List<ObjectPDF>
                            {
                                new RectanglePDF(10, 10, new ColorPDF(row["CodiceHexEmittente"].ToString())),
                                new FormattedTextAreaPDF
                                {
                                    Text = row["Prodotto"].ToString(),
                                    FontBold = true,
                                    FontColor = ColorPDF.Nero,
                                    DeltaX = 12F,
                                    FontSize=7,
                                }
                            };
                        tabellaProdottiPerArea.Cells[0, i].ValueObjectList.AddRange(listaOggetti);
                    }
                    else
                    {
                        var listaOggetti = new List<ObjectPDF>
                            {
                                new RectanglePDF(10, 10, ColorPDF.GetBy_AreaBisogno(row["CodiceAreaBisogno"].ToString())),
                                new FormattedTextAreaPDF
                                {
                                    Text = new AreaBisogno(row["CodiceAreaBisogno"].ToString()).Abbreviazione,
                                    FontBold = true,
                                    FontColor = ColorPDF.Bianco,
                                    DeltaX = 2.5F,
                                    FontSize=7,
                                },
                                new FormattedTextAreaPDF
                                {
                                    Text = row["Prodotto"].ToString(),
                                    FontBold = false,
                                    FontColor = ColorPDF.Nero,
                                    DeltaX = 12F,
                                    FontSize=7,
                                }
                            };
                        tabellaProdottiPerArea.Cells[0, i].ValueObjectList.AddRange(listaOggetti);
                    }

                    i++;
                }

                #endregion

                AddElement(tabellaProdottiPerArea);

                #endregion
            }
            #endregion
        }


        protected sealed override DataTable GetDataTable()
        {
            return null;
        }


        protected override DataSet GetDataSet()
        {
            var ds = new DataSet();
            ds.Tables.Add(new DataTable("patrimonio"));//0
            ds.Tables.Add(new DataTable("tipologiaEmittente"));//1            
            ds.Tables.Add(new DataTable("prodotti"));//2
            ds.Tables.Add(new DataTable("prodottiPerEmittente"));
            ds.Tables.Add(new DataTable("prodottiPerEmittenteFooter"));



            ds.Tables["patrimonio"].Columns.Add(new DataColumn("Descrizione", typeof(string)));
            ds.Tables["patrimonio"].Columns.Add(new DataColumn("Valore", typeof(string)));


            ds.Tables["tipologiaEmittente"].Columns.Add(new DataColumn("TipologiaEmittente", typeof(string)));
            ds.Tables["tipologiaEmittente"].Columns.Add(new DataColumn("Red", typeof(int)));
            ds.Tables["tipologiaEmittente"].Columns.Add(new DataColumn("Green", typeof(int)));
            ds.Tables["tipologiaEmittente"].Columns.Add(new DataColumn("Blue", typeof(int)));
            ds.Tables["tipologiaEmittente"].Columns.Add(new DataColumn("Controvalore", typeof(decimal)));
            ds.Tables["tipologiaEmittente"].Columns.Add(new DataColumn("Percentuale", typeof(decimal)));
            ds.Tables["tipologiaEmittente"].Columns.Add(new DataColumn("Ordinamento", typeof(int)));


            ds.Tables["prodotti"].Columns.Add(new DataColumn("CodiceAreaBisogno", typeof(string)));
            ds.Tables["prodotti"].Columns.Add(new DataColumn("OrdinamentoEmittente", typeof(int)));
            ds.Tables["prodotti"].Columns.Add(new DataColumn("CodiceHexEmittente", typeof(string)));

            ds.Tables["prodotti"].Columns.Add(new DataColumn("Prodotto", typeof(string)));
            ds.Tables["prodotti"].Columns.Add(new DataColumn("TipologiaEmittente", typeof(string)));
            ds.Tables["prodotti"].Columns.Add(new DataColumn("Emittente", typeof(string)));
            ds.Tables["prodotti"].Columns.Add(new DataColumn("Controvalore", typeof(decimal)));
            ds.Tables["prodotti"].Columns.Add(new DataColumn("Peso", typeof(decimal)));

            ds.Tables["prodotti"].Columns.Add(new DataColumn("FontBold", typeof(bool)));
            ds.Tables["prodotti"].Columns.Add(new DataColumn("ReplaceIfZero", typeof(string)));
            ds.Tables["prodotti"].Columns.Add(new DataColumn("CanLastRow", typeof(bool)));
            ds.Tables["prodotti"].Columns.Add(new DataColumn("IsTotal", typeof(bool)));

            var bNote = GetNote1().Length > 0;

            const string stringReplaceIfZero = " ";


            var dataDettaglioProdotti = datiSeiUnico.patrimonioUnit().patrimonioCasa.dettaglioProdotti.dettaglioProdotti.ToList();
            var dataDistribuzioneEmittente = datiSeiUnico.patrimonioUnit().patrimonioCasa.distribuzioneEmittente;
            var displayInfo = datiSeiUnico.displayInfos().emittentiDisplayInfos.ToList();

            #region patrimonio
            var testoPatrimonio = string.Format("Totale titoli obbligazionari (€){0}", bNote ? "*" : string.Empty);
            ds.Tables["patrimonio"].Rows.Add(testoPatrimonio, Helper.FormatCurrency(dataDistribuzioneEmittente.distribuzione.totaleRapprNoCC.ToString()));

            #endregion

            #region TipologiaEmittente


            if (presenzaGraficoBarre())
            {
                foreach (var itemDistribuzione in dataDistribuzioneEmittente.distribuzione.elencoSlice)
                {
                    if (itemDistribuzione.sliceRappresentabile)
                    {
                        //var colore = ColorPDF.Standard_Grigio_SfondoColonnaTabella; //TODO
                        //var ordinamento = 999;
                        var mapDisplayInfo = displayInfo.FirstOrDefault(l => l.key.Equals(itemDistribuzione.codice));
                        var colore = new ColorPDF(mapDisplayInfo.value.fill);
                        var ordinamento = mapDisplayInfo.value.order;
                        ds.Tables["tipologiaEmittente"].Rows.Add(itemDistribuzione.descrizione, colore.Red, colore.Green, colore.Blue, itemDistribuzione.ctv, itemDistribuzione.peso * 100, ordinamento);
                    }
                }
            }

            #endregion
            if (ds.Tables["tipologiaEmittente"].Rows.Count > 0)
                ds.Tables["tipologiaEmittente"].AsEnumerable().OrderBy(r => r.Field<int>("Ordinamento")).CopyToDataTable();

            #region Prodotti
            if (presenzaTabella())
            {
                foreach (var itemEmittente in dataDistribuzioneEmittente.distribuzionePerProdotti.listaProdotti)
                {
                    if (itemEmittente.rappresentabile)
                    {
                        var itemPosizione = dataDettaglioProdotti.FirstOrDefault(o => o.chiavePosizionePortafoglio.Equals(itemEmittente.chiaveAggregazione));
                        var mapDisplayInfo = displayInfo.Where(l => l.key == dataDistribuzioneEmittente.distribuzione.elencoSlice.FirstOrDefault(o => o.descrizione.Equals(itemEmittente.listaSlice[0].descrizione)).codice).ToArray();


                        ds.Tables["prodotti"].Rows.Add(
                            itemPosizione.area, //CodiceAreaBisogno 
                            mapDisplayInfo[0].value.order,//999, //OrdinamentoEmittente //TODO
                            mapDisplayInfo[0].value.fill,//"", //CodiceHexEmittente //TODO

                            itemPosizione.prodotto.nomeProdotto, //Prodotto
                            itemPosizione.prodotto.tipoEmittente, //TipologiaEmittente
                            itemPosizione.prodotto.emittente, //Emittente
                            itemEmittente.listaSlice[0].ctv, //Controvalore
                            itemEmittente.listaSlice[0].peso * 100, //Peso                      

                            false, //FontBold
                            " ", //ReplaceIfZero
                            true, //CanLastRow
                            false //IsTotal
                            );
                    }
                }
            }
            #endregion

            #region Prodotti raggruppati per Emittente

            //DataTable dtprodottiPerEmittente = ds.Tables["prodotti"].Clone();
            //dtprodottiPerEmittente.TableName = "prodottiPerEmittente";
            //ds.Tables.Add(dtprodottiPerEmittente);

            ds.Tables["prodottiPerEmittente"].Columns.Add(new DataColumn("CodiceAreaBisogno", typeof(string)));
            ds.Tables["prodottiPerEmittente"].Columns.Add(new DataColumn("OrdinamentoEmittente", typeof(int)));
            ds.Tables["prodottiPerEmittente"].Columns.Add(new DataColumn("CodiceHexEmittente", typeof(string)));

            ds.Tables["prodottiPerEmittente"].Columns.Add(new DataColumn("Prodotto", typeof(string)));
            ds.Tables["prodottiPerEmittente"].Columns.Add(new DataColumn("TipologiaEmittente", typeof(string)));
            ds.Tables["prodottiPerEmittente"].Columns.Add(new DataColumn("Emittente", typeof(string)));
            ds.Tables["prodottiPerEmittente"].Columns.Add(new DataColumn("Controvalore", typeof(decimal)));
            ds.Tables["prodottiPerEmittente"].Columns.Add(new DataColumn("Peso", typeof(decimal)));

            ds.Tables["prodottiPerEmittente"].Columns.Add(new DataColumn("FontBold", typeof(bool)));
            ds.Tables["prodottiPerEmittente"].Columns.Add(new DataColumn("ReplaceIfZero", typeof(string)));
            ds.Tables["prodottiPerEmittente"].Columns.Add(new DataColumn("CanLastRow", typeof(bool)));
            ds.Tables["prodottiPerEmittente"].Columns.Add(new DataColumn("IsTotal", typeof(bool)));


            var viewEmittenti = new DataView(ds.Tables["prodotti"]) { Sort = "OrdinamentoEmittente" };
            var dtEmittentiDistinct = viewEmittenti.ToTable(true, "TipologiaEmittente", "OrdinamentoEmittente", "CodiceHexEmittente");

            var tipologiaEmittente = string.Empty;

            #region Prodotti delle Risorse Finanziarie

            foreach (DataRow rowEmittente in dtEmittentiDistinct.Rows)
            {
                tipologiaEmittente = rowEmittente["TipologiaEmittente"].ToString();

                // subset di prodotti filtrati per TipologiaEmittente
                var dt = ds.Tables["prodotti"].Select("TipologiaEmittente='" + tipologiaEmittente + "'").OrderByDescending(r => r.Field<decimal>("Controvalore")).CopyToDataTable();

                #region riga con background grigio di intestazione per ogni area

                DataRow rowProdottiPerEmittente = ds.Tables["prodottiPerEmittente"].NewRow();

                rowProdottiPerEmittente["CodiceHexEmittente"] = rowEmittente["CodiceHexEmittente"];

                rowProdottiPerEmittente["Prodotto"] = rowEmittente["TipologiaEmittente"];
                rowProdottiPerEmittente["Controvalore"] = dt.AsEnumerable().Sum(x => x.Field<decimal>("Controvalore"));
                rowProdottiPerEmittente["Peso"] = dt.AsEnumerable().Sum(x => x.Field<decimal>("Peso"));

                rowProdottiPerEmittente["FontBold"] = true;
                rowProdottiPerEmittente["ReplaceIfZero"] = stringReplaceIfZero;
                rowProdottiPerEmittente["CanLastRow"] = false;
                rowProdottiPerEmittente["IsTotal"] = true;

                ds.Tables["prodottiPerEmittente"].Rows.Add(rowProdottiPerEmittente);

                #endregion

                // I prodotti della dtProdotti vengono mergiati sulla dtProdottiPerArea
                ds.Tables["prodottiPerEmittente"].Merge(dt);

            }

            #endregion

            #region Footer
            ds.Tables["prodottiPerEmittenteFooter"].Columns.Add(new DataColumn("Obbligazione", typeof(string)));
            ds.Tables["prodottiPerEmittenteFooter"].Columns.Add(new DataColumn("Controvalore", typeof(string)));
            ds.Tables["prodottiPerEmittenteFooter"].Columns.Add(new DataColumn("Peso", typeof(string)));

            ds.Tables["prodottiPerEmittenteFooter"].Rows.Add(
                                                        "Totale titoli obbligazionari",
                                                        Helper.FormatCurrency(dataDistribuzioneEmittente.distribuzione.totaleRapprNoCC.ToString()),
                                                        (dataDistribuzioneEmittente.distribuzione.pesoTot * 100).ToString()
                                                    );

            #endregion

            #endregion



            return ds;
        }



        public virtual bool isControvalore() { return !GetOption<Opzione3>().Valore; }
        public virtual bool presenzaGraficoBarre() { return GetOption<Opzione4_1>().Valore; }
        public virtual bool presenzaTabella() { return GetOption<Opzione4_2>().Valore; }
        public virtual string getTesto1()
        {
            return !isControvalore() ?
                Helper.ReplaceVariables(GetText().Rows[0]["testo1"].ToString(), base.EnvironmentFacade.ReportEnvironment) :
                Helper.ReplaceVariables(GetText().Rows[0]["testo2"].ToString(), base.EnvironmentFacade.ReportEnvironment);
        }
        public virtual string GetNote1()
        {
            string nota = GetText().Rows[0]["testo4"].ToString();

            var dataDistribuzioneEmittente = datiSeiUnico.patrimonioUnit().patrimonioCasa.distribuzioneEmittente;

            decimal GC = dataDistribuzioneEmittente.distribuzione.coverage * 100; // grado di copertura della distribuzione

            var listaValori = new List<decimal>()
            {
              (GC != 0 && GC != 100) ? GC : 0
            };

            var listaValoriStringa = new List<string>()
            {
                Helper.FormatPercentage(GC, 2)
            };

            return BuildDynamicNote(nota, listaValori, listaValoriStringa);
        }

        public virtual string getTesto2() { return GetText().Rows[0]["testo3"].ToString(); }
    }
}
